/*
 * Copyright (C) 2020-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CFG_HH_87F5A5D1859FC1086623BD7D5A6D2A32//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define CFG_HH_87F5A5D1859FC1086623BD7D5A6D2A32

#include "liblog/log_config.hh"

#include <boost/optional.hpp>
#include <boost/program_options.hpp>

#include <cstdint>
#include <list>
#include <map>
#include <stdexcept>
#include <string>
#include <vector>

namespace Fred {
namespace Zone {
namespace Client {
namespace Cfg {

class HelpRequested : public std::exception
{
public:
    explicit HelpRequested(std::string msg);
    const char* what() const noexcept override;
private:
    const std::string msg_;
};

struct Exception : std::runtime_error
{
    explicit Exception(std::string msg);
};

struct UnknownOption : Exception
{
    explicit UnknownOption(std::string msg);
};

struct MissingOption : Exception
{
    explicit MissingOption(std::string msg);
};

class Options
{
public:
    Options() = delete;
    Options(const Options&) = delete;
    Options(Options&&) = delete;
    Options& operator=(const Options&) = delete;
    Options& operator=(Options&&) = delete;

    static const Options& get();
    static const Options& init(int argc, const char* const* argv);

    struct Zone
    {
        boost::optional<std::string> zone_directory;
        boost::optional<std::string> backup_directory;
        boost::optional<std::uint64_t> max_changes_absolute;
        boost::optional<double> max_changes_relative;
        boost::optional<std::string> bind_config_file_name;
        boost::optional<std::string> header_file_name;
        boost::optional<std::string> footer_file_name;
        bool no_backups;
        boost::optional<std::string> post_hook_command;
        std::vector<std::string> post_hook_arguments;
    };
    struct Generic
    {
        boost::optional<std::string> config_file_name;
        Zone zone_common;
    } generic;
    struct Log
    {
        struct Console
        {
            LibLog::Level min_severity;
            LibLog::Sink::ConsoleSinkConfig::OutputStream output_stream;
            LibLog::ColorMode color_mode;
        };
        struct File
        {
            std::string file_name;
            LibLog::Level min_severity;
        };
        struct Syslog
        {
            std::string ident;
            LibLog::Level min_severity;
            int options;
            int facility;
        };
        boost::optional<Console> console;
        boost::optional<File> file;
        boost::optional<Syslog> syslog;
        bool log_grpc_library_events;
    } log;
    struct ZoneService
    {
        std::string connect_to;
    } zone_service;
    struct Generate
    {
        std::vector<std::string> zones;
    } generate;
    using ParticularZones = std::map<std::string, Zone>;
    ParticularZones zones;
    using NamesStorage = std::list<std::string>;
private:
    Options(int argc, const char* const* argv);
    NamesStorage names_storage_;
};

void handle_cli_args(int argc, char** argv);

}//namespace Fred::Zone::Client::Cfg
}//namespace Fred::Zone::Client
}//namespace Fred::Zone
}//namespace Fred

#endif//CFG_HH_87F5A5D1859FC1086623BD7D5A6D2A32
