/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "client/src/get_zones_to_generate.hh"

#include "client/src/unwrap_get_zone_list.hh"

#include <algorithm>
#include <stdexcept>

namespace Fred {
namespace Zone {
namespace Client {

namespace {

class GetZoneListVisitorImpl : public GetZoneListVisitor
{
public:
    GetZoneListVisitorImpl() = default;
    std::vector<std::string> zones() && noexcept;
private:
    void operator()(std::vector<std::string> zones) override;
    std::vector<std::string> zones_;
};

}//namespace Fred::Zone::Client::{anonymous}

std::vector<std::string> get_zones_to_generate(const Cfg::Options::Generate& to_generate, const Cfg::Options::ZoneService& zone_service)
{
    if (!to_generate.zones.empty())
    {
        return to_generate.zones;
    }
    GetZoneListVisitorImpl visitor{};
    unwrap_get_zone_list(zone_service, visitor);
    return std::move(visitor).zones();
}

namespace {

std::vector<std::string> GetZoneListVisitorImpl::zones() && noexcept
{
    return zones_;
}

void GetZoneListVisitorImpl::operator()(std::vector<std::string> zones)
{
    zones_ = std::move(zones);
}

}//namespace Fred::Zone::Client::{anonymous}

}//namespace Fred::Zone::Client
}//namespace Fred::Zone
}//namespace Fred
