/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef UNWRAP_GET_ZONE_DATA_HH_39B12BE31B4862CE6EF4CB7DF7B81962//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define UNWRAP_GET_ZONE_DATA_HH_39B12BE31B4862CE6EF4CB7DF7B81962

#include "client/src/cfg.hh"

#include <boost/asio/ip/address.hpp>
#include <boost/optional.hpp>

#include <cstdint>
#include <string>

namespace Fred {
namespace Zone {
namespace Client {

class GetZoneDataVisitor;

void unwrap_get_zone_data(const Cfg::Options::ZoneService& zone_service_options, const std::string& zone, GetZoneDataVisitor& visitor);

class GetZoneDataVisitor
{
public:
    struct Soa
    {
        struct RData
        {
            std::string m_name;
            std::string r_name;
            std::uint32_t serial;
            std::uint32_t refresh;
            std::uint32_t retry;
            std::uint32_t expire;
            std::uint32_t minimum;
        };
        std::string zone;
        RData r_data;
    };
    struct Common
    {
        boost::optional<std::uint32_t> ttl;
        Soa soa;
    };
    struct NsRData
    {
        std::string nameserver;
    };
    struct DsRData
    {
        std::uint16_t key_tag;
        std::uint8_t algorithm;
        std::uint8_t digest_type;
        std::string digest;
    };
    struct ARData
    {
        boost::asio::ip::address_v4 address;
    };
    struct AaaaRData
    {
        boost::asio::ip::address_v6 address;
    };
    struct Record
    {
        std::string owner;
        std::vector<NsRData> ns;
        std::vector<DsRData> ds;
        std::vector<ARData> a;
        std::vector<AaaaRData> aaaa;
    };
    struct NotFound
    {
        std::string message;
    };
    struct Exception
    {
        std::string message;
    };
private:
    virtual void operator()(Common common) = 0;
    virtual void operator()(Record record) = 0;
    virtual void operator()(NotFound exception) = 0;
    virtual void operator()(Exception exception) = 0;
    friend void unwrap_get_zone_data(const Cfg::Options::ZoneService&, const std::string&, GetZoneDataVisitor&);
};

}//namespace Fred::Zone::Client
}//namespace Fred::Zone
}//namespace Fred

#endif//UNWRAP_GET_ZONE_DATA_HH_39B12BE31B4862CE6EF4CB7DF7B81962
