/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "client/src/unwrap_get_zone_list.hh"

#include "fred_api/zone/service_zone_grpc.grpc.pb.h"

#include <grpc++/grpc++.h>

#include <algorithm>
#include <stdexcept>

namespace Fred {
namespace Zone {
namespace Client {

void unwrap_get_zone_list(const Cfg::Options::ZoneService& zone_service_options, GetZoneListVisitor& visitor)
{
    auto channel = grpc::CreateChannel(zone_service_options.connect_to, grpc::InsecureChannelCredentials());
    ::grpc::ClientContext ctx;
    ::google::protobuf::Empty request;
    Api::ZoneListReply reply;
    const auto result_status = Fred::Zone::Api::Zone::NewStub(channel)->get_zone_list(&ctx, request, &reply);
    if (!result_status.ok())
    {
        throw std::runtime_error{result_status.error_message()};
    }
    std::vector<std::string> zones;
    std::transform(reply.data().zones().begin(), reply.data().zones().end(), back_inserter(zones), [](const Api::ZoneName& zone_name)
    {
        return zone_name.fqdn();
    });
    visitor(std::move(zones));
}

}//namespace Fred::Zone::Client
}//namespace Fred::Zone
}//namespace Fred
