/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef ZONE_DATA_HH_5C357F7EFFC49ECF8064B9CD612AA88B//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define ZONE_DATA_HH_5C357F7EFFC49ECF8064B9CD612AA88B

#include <boost/asio/ip/address.hpp>
#include <boost/optional.hpp>

#include <cstdint>
#include <map>
#include <set>
#include <string>

namespace Fred {
namespace Zone {
namespace Client {

struct ZoneData
{
    std::string origin;
    boost::optional<std::uint32_t> ttl;
    struct SoaRecord
    {
        std::string m_name;
        std::string r_name;
        std::uint32_t serial;
        std::uint32_t refresh;
        std::uint32_t retry;
        std::uint32_t expire;
        std::uint32_t minimum;
    };
    SoaRecord soa_record;
    struct DomainData
    {
        std::set<std::string> nameservers;
        struct DsRecord
        {
            std::uint16_t key_tag;
            std::uint8_t algorithm;
            std::uint8_t digest_type;
            std::string digest;
            friend bool operator<(const DsRecord& lhs, const DsRecord& rhs) noexcept;
        };
        std::set<DsRecord> ds_records;
    };
    std::map<std::string, DomainData> domains;
    std::map<std::string, std::set<boost::asio::ip::address_v4>> a_records;
    std::map<std::string, std::set<boost::asio::ip::address_v6>> aaaa_records;
};

struct Diff
{
    Diff() noexcept;
    std::uint64_t unchanged;
    struct Change
    {
        std::uint64_t removed;
        std::uint64_t added;
    };
    boost::optional<Change> change;
};

template <typename T>
using DiffValues = boost::optional<std::pair<T, T>>;

struct ZoneDiff
{
    struct SoaRecordDiff
    {
        DiffValues<std::string> m_name;
        DiffValues<std::string> r_name;
        DiffValues<std::uint32_t> serial;
        DiffValues<std::uint32_t> refresh;
        DiffValues<std::uint32_t> retry;
        DiffValues<std::uint32_t> expire;
        DiffValues<std::uint32_t> minimum;
    };
    DiffValues<std::string> origin;
    DiffValues<boost::optional<std::uint32_t>> ttl;
    SoaRecordDiff soa_record;
    Diff domains;
    Diff nameservers;
    Diff ds_records;
    Diff a_records;
    Diff aaaa_records;
    void log() const;
    std::uint64_t absolute_change() const noexcept;
    double change_ratio() const noexcept;
};

ZoneDiff diff(const ZoneData& grpc, const ZoneData& file);

}//namespace Fred::Zone::Client
}//namespace Fred::Zone
}//namespace Fred

#endif//ZONE_DATA_HH_5C357F7EFFC49ECF8064B9CD612AA88B
