/*
 * Copyright (C) 2020-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/iface/tools/tools.hh"

#include "liblog/liblog.hh"

namespace Fred {
namespace Zone {
namespace Iface {
namespace Tools {

grpc::Status on_not_found(const Impl::Lib::NotFound& e)
{
    LIBLOG_INFO("NotFound: {}", e.what());
    return grpc::Status(grpc::StatusCode::NOT_FOUND, e.what());
}

grpc::Status on_runtime_error(const Impl::Lib::RuntimeError& e)
{
    LIBLOG_ERROR("RuntimeError caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNKNOWN, std::string{"RuntimeError: "} + e.what());
}

grpc::Status on_database_error(const LibPg::ConnectFailure& e)
{
    LIBLOG_ERROR("PostgreSQL ConnectFailure caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNAVAILABLE, std::string{"PostgreSQL ConnectFailure: "} + e.what());
}

grpc::Status on_database_error(const LibPg::ExecFailure& e)
{
    LIBLOG_ERROR("PostgreSQL ExecFailure caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::INTERNAL, std::string{"PostgreSQL ExecFailure: "} + e.what());
}

grpc::Status on_database_error(const LibPg::PgException& e)
{
    LIBLOG_ERROR("PostgreSQL exception caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::INTERNAL, std::string{"PostgreSQL exception: "} + e.what());
}

grpc::Status on_std_exception(const std::exception& e)
{
    LIBLOG_ERROR("std::exception caught: {}", e.what());
    return grpc::Status(grpc::StatusCode::UNKNOWN, "Unknown exception based on std::exception");
}

grpc::Status on_unknown_exception()
{
    LIBLOG_ERROR("unknown exception caught");
    return grpc::Status(grpc::StatusCode::UNKNOWN, "Unknown exception");
}

}//namespace Fred::Zone::Iface::Tools
}//namespace Fred::Zone::Iface
}//namespace Fred::Zone
}//namespace Fred
