/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/detail/base64.hh"

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {
namespace Detail {

static_assert(Base64Decoder::length() == 64, "Base64 decoder must be of 64 characters length");

static_assert(decode_base64_char('A') == 0, "'A' must be 0");
static_assert(decode_base64_char('Z') == 25, "'Z' must be 25");
static_assert(decode_base64_char('a') == 26, "'a' must be 26");
static_assert(decode_base64_char('z') == 51, "'z' must be 51");
static_assert(decode_base64_char('0') == 52, "'0' must be 52");
static_assert(decode_base64_char('9') == 61, "'9' must be 61");
static_assert(decode_base64_char('+') == 62, "'+' must be 62");
static_assert(decode_base64_char('/') == 63, "'/' must be 63");
static_assert(decode_base64_char('=') == 0xff, "'=' must be 0xff");

[[noreturn]] void throw_invalid_base64_character()
{
    throw_runtime_error([]() { return "invalid Base64 character"; });
}

[[noreturn]] void throw_unexpected_base64_character()
{
    throw_runtime_error([]() { return "unexpected Base64 character"; });
}

}//namespace Fred::Zone::Impl::Lib::Detail
}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
