/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/dnskey_record.hh"

#include "src/impl/lib/detail/base64.hh"

#include <utility>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

DnsKey::DnsKey(Flags flags,
               Protocol protocol,
               Algorithm algorithm,
               const PublicKey& public_key)
    : flags_{flags},
      protocol_{protocol},
      algorithm_{algorithm},
      public_key_{decode(*public_key)}
{ }

DnsKey::Flags DnsKey::flags() const
{
    return flags_;
}

DnsKey::Protocol DnsKey::protocol() const
{
    return protocol_;
}

DnsKey::Algorithm DnsKey::algorithm() const
{
    return algorithm_;
}

const DnsKey::PublicKeyWireFormat& DnsKey::public_key() const &
{
    return public_key_;
}

DnsKey::PublicKeyWireFormat DnsKey::public_key() &&
{
    return std::move(public_key_);
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
