/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DNSKEY_RECORD_HH_AD3C7C85F2FAC6320C5A91EA078766B4//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define DNSKEY_RECORD_HH_AD3C7C85F2FAC6320C5A91EA078766B4

#include "src/impl/lib/base64.hh"
#include "src/impl/lib/strong_type.hh"

#include <cstdint>
#include <vector>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

class DnsKey
{
public:
    using Flags = StrongType<std::uint16_t, struct DnsKeyFlagsTag_>;
    using Protocol = StrongType<std::uint8_t, struct DnsKeyProtocolTag_>;
    using Algorithm = StrongType<std::uint8_t, struct DnsKeyAlgorithmTag_>;
    using PublicKey = StrongType<Base64EncodedData, struct DnsKeyPublicKeyTag_>;
    using PublicKeyWireFormat = std::vector<std::uint8_t>;
    explicit DnsKey(Flags flags,
                    Protocol protocol,
                    Algorithm algorithm,
                    const PublicKey& public_key);
    Flags flags() const;
    Protocol protocol() const;
    Algorithm algorithm() const;
    const PublicKeyWireFormat& public_key() const &;
    PublicKeyWireFormat public_key() &&;
private:
    Flags flags_;
    Protocol protocol_;
    Algorithm algorithm_;
    PublicKeyWireFormat public_key_;
};

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred

#endif//DNSKEY_RECORD_HH_AD3C7C85F2FAC6320C5A91EA078766B4
