/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DS_RECORD_HH_4898DC9413D56E3893866794020AC74F//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define DS_RECORD_HH_4898DC9413D56E3893866794020AC74F

#include "src/impl/lib/absolute_fqdn.hh"
#include "src/impl/lib/dnskey_record.hh"
#include "src/impl/lib/strong_type.hh"

#include <cstdint>
#include <string>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

struct DsRecord
{
    using KeyTag = StrongType<std::uint16_t, struct DsRecordKeyTagTag_>;
    class DigestType
    {
    public:
        explicit DigestType(const std::string& digest_type);
        std::uint8_t get_value() const noexcept;
    private:
        friend bool operator<(DigestType, DigestType) noexcept;
        std::uint8_t value_;
    };
    using Digest = StrongType<std::string, struct DsRecordDigestTag_>;
    static KeyTag make_key_tag(const DnsKey& dns_key);
    static Digest make_digest(const AbsoluteFqdn& domain, const DnsKey& dns_key, DigestType digest_type);
};

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred

#endif//DS_RECORD_HH_4898DC9413D56E3893866794020AC74F
