/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/get_zones.hh"

#include "libpg/query.hh"

#include "liblog/liblog.hh"

#include <algorithm>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

ZoneList get_zones(const LibPg::PgTransaction& tx)
{
    using Fqdn = StrongType<std::string, struct FqdnTag_>;
    const auto dbres = exec(tx, LibPg::make_query("SELECT") << LibPg::item<Fqdn>() << "fqdn FROM zone");
    ZoneList zones;
    std::transform(dbres.begin(), dbres.end(), std::inserter(zones, std::begin(zones)), [](const auto& row)
            {
                return ZoneName{*row.template get<Fqdn>()};
            });
    return zones;
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
