/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef HASH_HH_DA5F0B9B6FA977AAFE6238D7E696D650//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define HASH_HH_DA5F0B9B6FA977AAFE6238D7E696D650

#include "src/impl/lib/strong_type.hh"

#include <cstddef>
#include <cstdint>
#include <string>
#include <vector>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

enum class HashFunction
{
    sha1,
    sha256,
    sha384
};

template <HashFunction> struct HashTag_;

template <HashFunction fnc>
using Hash = StrongType<std::vector<std::uint8_t>, HashTag_<fnc>>;

template <HashFunction fnc>
Hash<fnc> compute_hash(const void* data, std::size_t data_length);

using Sha1Hash = Hash<HashFunction::sha1>;
using Sha256Hash = Hash<HashFunction::sha256>;
using Sha384Hash = Hash<HashFunction::sha384>;

Sha1Hash compute_sha1_hash(const void* data, std::size_t data_length);
Sha256Hash compute_sha256_hash(const void* data, std::size_t data_length);
Sha384Hash compute_sha384_hash(const void* data, std::size_t data_length);

template <HashFunction fnc>
std::string to_string(const Hash<fnc>& data);

extern template Hash<HashFunction::sha1> compute_hash(const void*, std::size_t);
extern template Hash<HashFunction::sha256> compute_hash(const void*, std::size_t);
extern template Hash<HashFunction::sha384> compute_hash(const void*, std::size_t);

extern template std::string to_string(const Hash<HashFunction::sha1>&);
extern template std::string to_string(const Hash<HashFunction::sha256>&);
extern template std::string to_string(const Hash<HashFunction::sha384>&);

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred

#endif//HASH_HH_DA5F0B9B6FA977AAFE6238D7E696D650
