/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/zone_list.hh"

#include "src/impl/lib/get_zones.hh"

#include "src/util/util.hh"

#include "liblog/liblog.hh"

namespace Fred {
namespace Zone {
namespace Impl {

Lib::ZoneList zone_list()
{
    LIBLOG_DEBUG("enter");
    try
    {
        auto tx = Util::make_transaction();
        auto result = Lib::get_zones(tx);
        commit(std::move(tx));
        LIBLOG_DEBUG("successfully done");
        return result;
    }
    catch (const LibPg::ExecFailure& e)
    {
        LIBLOG_WARNING("SQL exec failure: {}", e.what());
        throw;
    }
    catch (const LibPg::ConnectFailure& e)
    {
        LIBLOG_WARNING("DB connect failure: {}", e.what());
        throw;
    }
    catch (const std::exception& e)
    {
        LIBLOG_WARNING("std::exception caught: {}", e.what());
        throw;
    }
}

}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
