/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef CFG_HH_C3ED8EA3D73FE86CF0FEE3C3107A0B50//date "+%s.%N"|md5sum|tr "[a-f]" "[A-F]"
#define CFG_HH_C3ED8EA3D73FE86CF0FEE3C3107A0B50

#include "liblog/log_config.hh"

#include <boost/optional.hpp>
#include <boost/program_options.hpp>

#include <cstdint>
#include <stdexcept>
#include <string>

namespace Test {
namespace Cfg {

class HelpRequested : public std::exception
{
public:
    explicit HelpRequested(std::string msg);
    const char* what() const noexcept override;
private:
    const std::string msg_;
};

struct Exception : std::runtime_error
{
    explicit Exception(std::string msg);
};

struct UnknownOption : Exception
{
    explicit UnknownOption(std::string msg);
};

struct MissingOption : Exception
{
    explicit MissingOption(std::string msg);
};

class Options
{
public:
    Options() = delete;
    Options(const Options&) = delete;
    Options(Options&&) = delete;
    Options& operator=(const Options&) = delete;
    Options& operator=(Options&&) = delete;

    static const Options& get();
    static const Options& init(int argc, const char* const* argv);

    boost::optional<std::string> config_file_name;
    struct Log
    {
        struct Console
        {
            LibLog::Level min_severity;
            LibLog::Sink::ConsoleSinkConfig::OutputStream output_stream;
            LibLog::ColorMode color_mode;
        };
        struct File
        {
            std::string file_name;
            LibLog::Level min_severity;
        };
        boost::optional<Console> console;
        boost::optional<File> file;
        bool log_grpc_library_events;
    } log;
    struct ZoneService
    {
        std::string connect_to;
    } zone_service;
    struct Zone
    {
        std::string fqdn;
    } zone;
    struct ZoneFile
    {
        std::string file_name;
    } zone_file;
private:
    Options(int argc, const char* const* argv);
};

void handle_cli_args(int argc, char** argv);

}//namespace Test::Cfg
}//namespace Test

#endif//CFG_HH_C3ED8EA3D73FE86CF0FEE3C3107A0B50
