/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "setup.hh"

#include "src/impl/lib/hash.hh"

#include "src/cfg.hh"

#include <boost/test/unit_test.hpp>

#include <array>

BOOST_AUTO_TEST_SUITE(TestHash)

namespace {

constexpr auto example = std::array<char, 3>{ 'a', 'b', 'c' };
constexpr auto example_sha1_hash   = "A9993E364706816ABA3E25717850C26C9CD0D89D";
constexpr auto example_sha256_hash = "BA7816BF8F01CFEA414140DE5DAE2223B00361A396177A9CB410FF61F20015AD";
constexpr auto example_sha384_hash = "CB00753F45A35E8BB5A03D699AC65007272C32AB0EDED1631A8B605A43FF5BED8086072BA1E7CC2358BAECA134C825A7";

}//namespace {anonymous}

BOOST_AUTO_TEST_CASE(compute_sha1_hash)
{
    const auto hash = Fred::Zone::Impl::Lib::compute_sha1_hash(example.data(), example.size());
    const auto hexa_hash = Fred::Zone::Impl::Lib::to_string(hash);
    BOOST_CHECK_EQUAL(hexa_hash, example_sha1_hash);
}

BOOST_AUTO_TEST_CASE(compute_sha256_hash)
{
    const auto hash = Fred::Zone::Impl::Lib::compute_sha256_hash(example.data(), example.size());
    const auto hexa_hash = Fred::Zone::Impl::Lib::to_string(hash);
    BOOST_CHECK_EQUAL(hexa_hash, example_sha256_hash);
}

BOOST_AUTO_TEST_CASE(compute_sha384_hash)
{
    const auto hash = Fred::Zone::Impl::Lib::compute_sha384_hash(example.data(), example.size());
    const auto hexa_hash = Fred::Zone::Impl::Lib::to_string(hash);
    BOOST_CHECK_EQUAL(hexa_hash, example_sha384_hash);
}

BOOST_AUTO_TEST_SUITE_END()//TestHash
